const io = require("socket.io");
const AuthService = require("./AuthService");
const UpService = require("./UpService");
const UpdaterSocket = require("./UpdaterSocket");
const MonitoringSocket = require("./MonitoringSocket");

class SocketService {
    getUpdaterSocket() {
        return UpdaterSocket;
    }
    getMonitoringSocket() {
        return MonitoringSocket;
    }
    async start(http) {
        this.io = io(http, { cors: "*" });

        UpdaterSocket.setSocketIo(this.io);
        MonitoringSocket.setSocketIo(this.io);
        await MonitoringSocket.loadInitialParams();

        this.io.on("connection", (socket) => {
            try {
                const { log: info, agent, version, cdfilial } = this.getSocketHeaderInfo(socket);
                const { page } = this.getQueryParams(socket);
                console.log(`OnConnection (${info}): ${socket.id}`);
                switch (agent) {
                    case "updater":
                        UpdaterSocket.validateVersionCompatibility(socket, version, info);
                        UpdaterSocket.joinDefaultRoom(socket);
                        break;
                    case "client":
                        MonitoringSocket.hasConsumersForMonitoring().then( hasConsumers => {
                            if(hasConsumers) {
                                MonitoringSocket.startMonitoring([socket]);
                                MonitoringSocket.connectedClient([cdfilial]);
                            }
                        });
                        MonitoringSocket.joinDefaultRoom(socket);
                        break;
                    default:
                        if (page === 'monitoring') {
                            MonitoringSocket.joinRooms(socket);
                            MonitoringSocket.connectedUser();
                            MonitoringSocket.startMonitoring();
                        } else if (page === 'updates') {
                            UpdaterSocket.joinRooms(socket);
                        }
                }

                /** Garante que o portal está conectado */

                socket.on("login", (data) => {
                    this.login(socket, data);
                });

                /**
                 * Exportação de vendas
                 */
                socket.on("iniciaExportacao", (data) => {
                    try {
                        data.payload = JSON.parse(data.payload);
                        UpService.setInsereServidor(
                            data.CDFILIAL,
                            data.CDCAIXA,
                            data.CDLOJA,
                            data.payload
                        );
                    } catch (e) {}
                });

                socket.on("disconnect", async () => {
                    let agent = this.getSocketHeaderInfo(socket).agent.toLowerCase();
                    if( agent === 'client') {
                        MonitoringSocket.connectedClient([socket.CDFILIAL], 'N');
                    } else if ( agent === 'updater' ) {
                        UpdaterSocket.connectedUpdater([socket.CDFILIAL], false);
                    } else { // master
                        MonitoringSocket.disconnectedUser();
                    }
                });

                MonitoringSocket.registerEvents(socket, agent);
                UpdaterSocket.registerEvents(socket, agent);
            } catch (error) {
                console.log(`ERR: ${error.message}`);
            }
        });
    }

    getQueryParams(socket) {
        const defaults = {
            page: '',
        };
        const { query } = socket.handshake;
        return ({
            ...defaults,
            ...query
        });
    }

    getSocketHeaderInfo(socket) {
        let { headers } = socket.handshake;
        const defaults = {
            agent: "",
            cdfilial: "",
            hostname: "",
            version: "",
            "user-agent": "",
            "x-real-ip": socket.handshake.address,
        };
        let { agent, cdfilial, hostname, version, ...others } = {
            ...defaults,
            ...headers,
        };
        let ip = others["x-real-ip"];
        let userAgent = others["user-agent"];
        let log = [agent, cdfilial, hostname, ip]
            .filter((value) => value)
            .join(" | ");
        return { log, agent, cdfilial, hostname, version, ip, userAgent };
    }

    login(socket, data) {
        // Não é uma filial que está conectando e sim um portal
        if (!data.CDFILIAL) {
            AuthService.autenticar(data.email, data.password)
                .then((result) => {
                    socket.email = data.email;
                    this.io.to(socket.id).emit("confirmLogin", {
                        error: false,
                        message: "Você está conectado",
                        result: result,
                    });

                    if (!data.slave && !data.updater) {
                        MonitoringSocket.connectedClient();
                        UpdaterSocket.connectedUpdater();
                    }
                })
                .catch((error) => {
                    console.log(error);
                    this.io
                        .to(socket.id)
                        .emit("message", "Você não é um usuário autorizado!");
                    socket.disconnect();
                });
        } else {
            // Uma filial se coneta ao master (Updater ou Monitoring)
            AuthService.autenticar(data.email, "", data.CDFILIAL)
                .then(() => {
                    socket.email = data.email;
                    socket.CDFILIAL = data.CDFILIAL;

                    // IF -> Conexão de client do Updater
                    // ELSE -> Conexão de client do Monitoring
                    if (data.updater) {
                        UpdaterSocket.connectedUpdater([socket.CDFILIAL]);
                    } else {
                        MonitoringSocket.connectedClient([socket.CDFILIAL]);
                    }
                })
                .catch((error) => {
                    console.log(error);
                    socket.emit("message", "Você não é um usuário autorizado!");
                    socket.disconnect();
                });
        }
    }

}

module.exports = new SocketService();