const updaterModel = require("../model/updaterModel");
const base64 = require("../functions/Base64");
const semver = require("semver");
const supportedUpdaterVersion = "1.0.0";
const SocketRooms = require("./SocketRooms");
const MonitoringSocket = require('./MonitoringSocket');

class UpdaterSocket {
    clients = [];
    setSocketIo(io) {
        this.io = io;
    }
    getSocketIo() {
        return this.io;
    }
    async connectedUpdater(UNITS = null, CONNECTED = true) {
        UNITS = Array.isArray(UNITS) ? UNITS : await this.getUpdaters().then( sockets => {
            return sockets.map( socket => socket.CDFILIAL);
        });
        this.getRoom(SocketRooms.master.updatesPage).emit("updaterConnected", {UNITS, CONNECTED});
    }
    joinDefaultRoom( socket ) {
        socket.join(SocketRooms.updater.default);
    }
    joinRooms( socket ) {
        socket.join(SocketRooms.master.default);
        socket.join(SocketRooms.master.updatesPage);
    }
    validateVersionCompatibility(socket, version, info) {
        if (semver.gt(supportedUpdaterVersion, version)) {
            socket.emit("invalidVersion", {
                supportedUpdaterVersion: supportedUpdaterVersion,
                updaterVersion: version,
            });

            console.log(
                `Disconnecting: (${info}): ${socket.id} - Versão do updater inferior a ${supportedUpdaterVersion}`
            );
            socket.disconnect();
            throw new Error("UPDATER_VERSION_INCOMPATIBLE");
        } else {
            socket.emit("validVersion");
        }
    }
    getRoom(roomName) {
        return this.getSocketIo().to(roomName);
    }
    async getUpdaters() {
        return await this.getRoom(SocketRooms.updater.default).fetchSockets();
    }
    async getMasterFrontendClients(id = null){
        const sockets = await this.getRoom(SocketRooms.master.default).fetchSockets();
        return id ? sockets.find( sckt => sckt.id === id) : sockets;
    }
    async getSocketUpdaterByUnit(CDFILIAL) {
        return this.getUpdaters()
            .then((allSockets) => {
                const updaterUnitSocket = allSockets.find((socket) => {
                    return socket.CDFILIAL == CDFILIAL;
                });
                return updaterUnitSocket;
            });
    }
    getUnitBySocketUpdater( socket ) {
        const { headers = {} } = socket.handshake;
        const { cdfilial = 'Unidade não informada no socket-client' } = headers;
        return cdfilial;
    }
    registerUpdaterSocketEvents(socket) {
        socket.on("updateConfigResult", (resultConfigIni) => {
            socket.emit(
                "updateConfigIniFinish",
                resultConfigIni
            );
        });

        socket.on("updaterLogResponse", async (logs) => {
            if(!logs.CDFILIAL) return;
            const { 
                CDFILIAL, 
                masterSocketId, 
                usingLatestLines = false, 
                ...responseLogs 
            } = logs;
            const roomLogsUnit = this.getRoom(SocketRooms.updater.log(CDFILIAL));
            if(usingLatestLines) {
                const masterSockets = await this.getMasterFrontendClients();
                const targetSocket = masterSockets.find( socket => socket.id === masterSocketId);
                if(targetSocket) targetSocket.emit('updaterLog', responseLogs);
            } else {
                roomLogsUnit.emit('updaterLog', responseLogs);
            }
        });

        socket.on("updaterResultInProgress", (result) => {
            this.getRoom(SocketRooms.master.updatesPage).emit(
                "updaterInProgress",
                result
            );
        });

        socket.on("updaterResultFinish", (result) => {
            this.getRoom(SocketRooms.master.updatesPage).emit("updaterFinish", result);
        });

        socket.on("updaterChangeServices", (result) => {
            const CDFILIAL = this.getUnitBySocketUpdater(socket);
            switch(result.operation.toUpperCase()) {
                case 'INSTALL':
                    MonitoringSocket.addServiceToMonitoring(CDFILIAL, result.services);
                    break;
                case 'REMOVE':
                    MonitoringSocket.removeServiceToMonitoring(CDFILIAL, result.services);
                    break;
                default:
                    console.log('operation not mapped: ', result);
            }
        })
    }
    registerMasterSocketEvents(socket) {
        socket.on("updateConfigInit", async (CDFILIAL) => {
            const socketUpdater = await this.getSocketUpdaterByUnit( CDFILIAL );
            if (socketUpdater) {
                try {
                    const { rows } = await updaterModel.getConfigIniByUnit({
                        P_CDFILIAL: CDFILIAL,
                    });

                    if (rows.length === 0) {
                        throw new Error(
                            `Não foi encontrado o registro do arquivo de configuração para a filial ${CDFILIAL}`
                        );
                    }

                    const configIni = rows[0].CONFIG;
                    const configIniDecode = base64.decode(configIni);

                    socketUpdater.emit("updateConfig", configIniDecode);
                } catch (error) {
                    socket.emit(
                        "updateConfigIniFinish",
                        {
                            error: true,
                            message: error.message,
                        }
                    );
                }
            } else {
                socket.emit(
                    "updateConfigIniFinish",
                    {
                        error: true,
                        message:
                            "Arquivo de configuração não pode ser atualizado na loja. Não existe conexão ativa com a loja no momento.",
                    }
                );
            }
        });

        socket.on("getUpdaterLog", async (params) => {
            const updaterUnitSocket = await this.getSocketUpdaterByUnit(
                params.CDFILIAL
            );
            if (updaterUnitSocket) {
                socket.join(SocketRooms.updater.log(params.CDFILIAL));
                updaterUnitSocket.emit("getUpdaterLog", {...params, masterSocketId: socket.id});
            }
        });

        socket.on('stopUpdaterLog', async (params) => {
            if(!params.CDFILIAL) return;
            const roomName = SocketRooms.updater.log(params.CDFILIAL);
            socket.leave(roomName);
            const roomLogs = this.getRoom(roomName);
            const connectedSockets = await roomLogs.fetchSockets();
            if(!connectedSockets.length) {
                const updaterSocket = await this.getSocketUpdaterByUnit(params.CDFILIAL);
                if(updaterSocket) updaterSocket.emit('unwatchLogFile', params);
            }
        })

        socket.on("execUpdater", async (params) => {
            const updaterSocketUnit = await this.getSocketUpdaterByUnit(params.CDFILIAL);
            if (updaterSocketUnit) {
                // Envia requisição de atualização para a loja
                updaterSocketUnit.emit("updater", params);
            } else {
                console.log(
                    "CLIENT DO UPDATER NÃO ENCONTRADO NA FILIAL: " +
                        params.CDFILIAL
                );
            }
        });
    }
    registerEvents(socket, agent) {
        if(agent === 'updater') {
            this.registerUpdaterSocketEvents(socket);
        } else if(agent !== 'client') {
            this.registerMasterSocketEvents(socket);
        }
    }
}

module.exports = new UpdaterSocket();